/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PROTO_ICMP6_HH_
# define PROTO_ICMP6_HH_

extern "C"
{
# include <inttypes.h>
}

/// ICMPv6 related compounds.
namespace icmp6
{

  /// ICMPv6 message types.
  namespace type
  {

    /// ICMPv6 message type.
    enum type
      {
	dest_unreachable	= 1,
	packet_too_big		= 2,
	time_exceeded		= 3,
	param_problem		= 4,
	priv_exp_1		= 100,
	priv_exp_2		= 101,
	reserved_1		= 127,
	echo_request		= 128,
	echo_reply		= 129,
	router_sol		= 133,
	router_adv		= 134,
	priv_exp_3		= 200,
	priv_exp_4		= 201,
	reserved_2		= 255
      };

  } // End of namespace icmp6::type;

  /// ICMPv6 header.
  struct header
  {

    uint8_t	type;
    uint8_t	code;
    uint16_t	checksum;

  } __attribute__((__packed__));

} // End of namespace icmp6.

#endif // ! PROTO_ICMP6_HH_
