/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PROTO_IP6_HH_
# define PROTO_IP6_HH_

extern "C"
{
# include <inttypes.h>
}

# include <wipal/tool/addr.hh>
# include <wipal/proto/ip4.hh>

/// IPv6 related compounds.
namespace ip6
{

  /// IPv6 address.
  typedef tool::addr<uint16_t, 8, true, ':'>	addr;

  /// IPv6 protocols (compatible with IPv4).
  namespace proto
  {

    using namespace ip4::proto;

  } // End of namespace ip6::proto.

  /// IPv6 header.
  struct header
  {
    /// IPv6 addresses index.
    enum addr_index
      {
	sa	= 0,	///< Source address.
	da	= 1,	///< Destination address.
      };

    union
    {
      // Use a bitmask to retrieve the relevant bits.
      uint32_t	version;
      uint32_t	trafficclass;
      uint32_t	flowlabel;
    };

    uint16_t	payloadlen;
    uint8_t	nextheader;
    uint8_t	hoplimit;

    addr	addrs[2];

  } __attribute__((__packed__));

} // End of namespace ip6.

#endif // ! PROTO_IP6_HH_
