/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_MICROSECONDS_HH_
# define TOOL_MICROSECONDS_HH_

extern "C"
{
# include <stdint.h>
# include <sys/time.h>
}
# include <gmpxx.h>

namespace tool
{

  /// Represent an arbitrary number of %microseconds.
  struct microseconds: public mpz_class
  {
    typedef mpz_class	mpz_type;

    /// Construct from a 64-bit unsigned integer.
    microseconds(const uint64_t);

    /// Construct from a @c timeval struct.
    microseconds(const struct timeval& tv);


    /// @name Interaction with GMP
    //@{

    /// Construct from a GMP bignum.
    microseconds(const mpz_type&);

    /// Construct from a GMP expression.
    template <class T1, class T2>
    microseconds(const __gmp_expr<T1, T2>& e);

    //@}

    /// Convert the microsecond count into a suitable Prism header MAC time.
    uint32_t		get_prism_mactime() const;

    /// Convert the microsecond count into a suitable Prism header host time.
    uint32_t		get_prism_hosttime() const;

    /// Convert the microsecond count into a 64 bit integer.
    uint64_t		get_uint64() const;

    /**
     * Divide the microsecond count by @c d and return the result.
     *
     * A negative microsecond count or a result that does not fit into
     * an unsigned long yields undefined behavior.
     */
    unsigned		get_div_by(unsigned d) const;

    /// Convert the microsecond count into a suitable @c timeval structure.
    struct timeval	get_timeval() const;
  };

} // End of namespace tool.

# include "microseconds.hxx"

#endif // TOOL_MICROSECONDS_HH_
