/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_ADDR_MAPPING_HH_
# define WIFI_ADDR_MAPPING_HH_

# include <wipal/tool/mapping.hh>
# include <wipal/wifi/addr.hh>

namespace wifi
{

  namespace internals
  {

    /// Factory for @c addr_mapping.
    struct addr_mapping_factory
    {
      unsigned	new_value(const addr&				,
			  const std::map<addr, unsigned>&	) const;

      void	value_loaded(const addr&			,
			     const unsigned&			,
			     const std::map<addr, unsigned>&	) const;
    };

  } // End of namespace wifi::internals.

  /**
   * IEEE 802.11 address to integer mapping designed to be stored into
   * a database file.
   *
   * Load the mapping from the file at creation when the file exists.
   * Store it at deletion so it could be used later, or loaded in
   * a database using a LOAD DATA INFILE SQL command.
   */
  typedef
    tool::mapping<addr, unsigned, internals::addr_mapping_factory>
    addr_mapping;

} // End of namespace wifi.

# include "addr_mapping.hxx"

#endif // ! WIFI_ADDR_MAPPING_HH_
