/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_STATS_MISSED_ESTIMATIONS_HH_
# define WIFI_FRAME_STATS_MISSED_ESTIMATIONS_HH_

# include <map>

# include <wipal/wifi/frame/stats/simple_counters.hh>
# include <wipal/tool/microseconds.hh>
# include <wipal/wifi/addr.hh>

namespace wifi
{

  namespace frame
  {

    namespace stats
    {

      struct missed_estimations: public module
      {
	template <class H, class T>
	missed_estimations(const module_list<H, T>* ml);

	template <class Frame>
	void
	account_beacon(const Frame&		frame,
		       const addr* const&	ap,
		       const addr* const&	bss,
		       const optional_string&	ssid,
		       const bool&		ibss);

	void		account_gap(const unsigned& gap_length);
	void		account_miss();

	std::ostream&	print(std::ostream& o) const;

      private:
	enum
	  {
	    // FIXME: Theoretically APs might use other beacon
	    // FIXME: intervals, but we do not handle that.

	    beacon_interval	= 102400
	  };

	typedef std::map<addr, tool::microseconds>  beacons_map;

	beacons_map		beacons_;
	unsigned		missed_beacons_;
	unsigned		missed_;
	const simple_counters*	sc_;
      };

    } // End of namespace wifi::frame::stats.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

# include "missed_estimations.hxx"

#endif // ! WIFI_FRAME_STATS_MISSED_ESTIMATIONS_HH_
