/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_STATS_MODULE_HH_
# define WIFI_FRAME_STATS_MODULE_HH_

# include <string>
# include <cstddef>
extern "C"
{
# include <sys/time.h>
}
# include <boost/optional.hpp>

# include <wipal/wifi/frame/stats/abstract_stats.hh>
# include <wipal/wifi/addr.hh>
# include <wipal/tool/microseconds.hh>
# include <wipal/wifi/frame/frame.hh>

namespace wifi
{

  namespace frame
  {

    namespace stats
    {

# define WP_module_methods_declaration					\
      typedef boost::optional<std::string>		optional_string; \
									\
      void	account_addr(const addr& a);				\
									\
      template <class Frame>						\
      void								\
      account_beacon(const Frame&		frame,			\
		     const addr* const&		ap,			\
		     const addr* const&		bss,			\
		     const optional_string&	ssid,			\
		     const bool&		ibss);			\
									\
      void	account_duplicate();					\
      void	account_expired();					\
									\
      template <class Frame>						\
      void								\
      account_frame(const Frame&		frame,			\
		    const size_t&		len80211,		\
		    const type::frame_type&	type,			\
		    const unsigned&		subtype,		\
		    const bool&			retx,			\
		    const addr* const&		receiver,		\
		    const addr* const&		transmitter,		\
		    const bool&			from_ap);		\
									\
      void	account_gap(const unsigned& gaplen);			\
      void	account_miss();						\
      void	account_out_of_order();					\
      void	account_seq_anomaly();					\
									\
      void	restart()

      struct module: public abstract_stats
      {
	WP_module_methods_declaration;
      };

      template <class Head, class Tail>
      struct module_list: protected Head, protected Tail
      {
	module_list();

	template <class ML>
	module_list(const ML* other_modules);

	WP_module_methods_declaration;

	std::ostream&				print(std::ostream& os) const;

	template <class T>	T&		get();
	template <class T>	const T&	get() const;
      };

# undef WP_module_methods_declaration

      template <class Head, class Tail>
      std::ostream&
      operator << (std::ostream& os, const module_list<Head, Tail>& ml);

    } // End of namespace wifi::frame::stats.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

# include "module.hxx"

#endif // ! WIFI_FRAME_STATS_MODULE_HH_
