/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008, 2009  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#include <iostream>
#include <stdexcept>
#include <cstdlib>
#include <ctime>
#include <boost/foreach.hpp>

#include <wipal/tool/args.hh>
#include <wipal/pcap/list_of_traces.hh>
#include <wipal/wifi/frame/filter/anonymizer.hh>

int
main(int argc, char *const argv[])
{
  try
    {
      namespace opt = tool::opt;
      namespace args = tool::args;
      namespace def = args::def;

      typedef pcapxx::descriptor<>	descriptor;
      typedef pcapxx::list_of_traces<>	list_of_traces;

      opt::list	opts ((opt::name ("misc") -= opt::list
		       ((opt::name ("mac_map") -= def::mac_map (),
			 opt::name ("essid_map") -= def::essid_map ()))));

      args::parse(argc, argv, opts, args::count (1), args::count (1));
      srand(time(NULL));

      const args::input&	input = opt::get<args::input>(opts["proc"]
							      ["input"]);
      list_of_traces		traces (input[0]);

      if (traces.empty())
	throw std::invalid_argument ("Empty list of traces");

      const args::input&	output = opt::get<args::output>(opts["proc"]
								["output"]);
      pcapxx::dumper		dump = traces.get(0).dumper(output[0]);

      const descriptor::link_type	lt = traces.get(0).linktype();

      args::check_options(opts, lt);
      args::ensure_80211(lt);
      traces.expect(lt);

      switch (lt)
	{

#define WP_case(R, Unused, LT)						\
	case descriptor::BOOST_PP_TUPLE_ELEM(2, 0, LT):			\
	  {								\
	    using opt::get;						\
									\
	    typedef BOOST_PP_TUPLE_ELEM(2, 1, LT)	phy_header;	\
	    typedef list_of_traces::const_iterator	iterator;	\
	    typedef wifi::frame::filter::anonymizer<iterator, phy_header> \
							anonymizer;	\
	    typedef wifi::frame::anon::default_policy	anon_policy;	\
									\
	    const opt::list&	m = get<opt::list>(opts["misc"]);	\
									\
	    anonymizer a (traces.begin(), traces.end(),			\
			  anon_policy (get<std::string>(m["mac_map"]),	\
				       get<std::string>(m["essid_map"]))); \
									\
	    a.for_each(dump);						\
	  }								\
	break;

	  BOOST_PP_SEQ_FOR_EACH(WP_case, ~, WP_LINKTYPES)

	default:
	    args::unsupported_linktype(lt);
	    break;
	}
    }
  catch (std::exception& e)
    {
      std::cerr << "ERROR: " << e.what() << '.' << std::endl;
      return EXIT_FAILURE;
    }
}
