/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#include <fstream>
#include <sstream>

#include "wit_importer.hh"

namespace wit
{

  importer::importer(const std::string& prefix): last_channel_ (0)
  {
    frames_data_.reserve(channels_max);
    mappings_.reserve(channels_max);

    for (unsigned i = 1; i <= channels_max; ++i)
      {
	const std::string	frdata_fname = frames_data_filename(prefix, i);

	frames_data_.push_back(frdata_fname);
	mappings_.push_back(mapping_filename(prefix, i));

	std::ifstream	ifs (frdata_fname.c_str());

	if (not ifs)
	  last_id_[i - 1] = 0;
	else
	  {
	    enum { linelength_max = 1024 };

	    ifs.seekg(-linelength_max, std::ifstream::end);

	    std::string last;

	    while (true)
	      {
		char	line[linelength_max];

		ifs.getline(line, linelength_max);
		if (not ifs or ifs.eof())
		  break;
		last = std::string(line);
	      }

	    std::istringstream is (last);
	    is >> last_id_[i - 1];
	  }
      }
  }

  const std::string
  importer::db_null = "\\N";

} // End of namespace wit.
